const express = require("express");
const router = express.Router();
const BlogController = require("../controllers/blog");
const isAuth = require("../middleware/requirelogin");
const multer = require("multer");
const multerS3 = require("multer-s3");
const aws = require("aws-sdk");

aws.config.update({
  secretAccessKey: `${process.env.AWS_SEC}`, //`${process.env.AWS_SEC}`,
  accessKeyId: `${process.env.AWS_KEY}`, //`${process.env.AWS_KEY}`,
  region: "ap-south-1",
});

const s3 = new aws.S3();

const upload = multer({
  storage: multerS3({
    s3: s3,
    acl: "public-read",
    bucket: "cantilever-blog-images",
    metadata: function (req, file, cb) {
      cb(null, { fieldName: file.originalname });
    },

    key: function (req, file, cb) {
      cb(null, file.originalname);
    },
  }),
});

router.post("/addBlog", upload.single("blog_image"), BlogController.addBlog);

router.get("/getAllBlogs", BlogController.getAllBlogs);

router.get("/blog/:id", BlogController.getSingleBlog);

router.post("/deleteBlog/", BlogController.deleteBlog);

router.post("/editBlog/", BlogController.editBlog);

router.post("/commentBlog/:id", isAuth, BlogController.commentBlog);

module.exports = router;
